package no.skippy.agent;

import java.util.HashMap;
import java.util.Map;

import no.skippy.agent.event.AfterTurnEvent;
import no.skippy.agent.event.RoundEndedEvent;
import no.skippy.agent.event.TurnEvent;
import no.skippy.agent.util.Storage;

import org.apache.commons.logging.Log;
import org.jcurl.core.log.JCLoggerFactory;

/**
 * The basic agent class that you will extend to create your own curling agent.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public abstract class BasicAgent implements Agent {

	private Map<String, Object> resource = new HashMap<String, Object>();
	private Log logger = JCLoggerFactory.getLogger(getClass());

	/**
	 * Construct a new Curling Agent.
	 */
	public BasicAgent() {}

	/** {@inheritDoc} */
	public void afterTurn(AfterTurnEvent event) {}

	/**
	 * {@inheritDoc} </p> By default {@link BasicAgent#getAgentName()} returns the
	 * string representation of the agent.
	 */
	public String getAgentName() {
		return toString();
	}

	/** {@inheritDoc} */
	public void onLoose(RoundEndedEvent event) {}

	/** {@inheritDoc} */
	public void onTie(RoundEndedEvent event) {}

	/** {@inheritDoc} */
	public void onTurn(TurnEvent event) {}

	/** {@inheritDoc} */
	public void onWin(RoundEndedEvent event) {}

	/** {@inheritDoc} */
	public void loadResources(Storage service) {}
	
	/** {@inheritDoc} */
	public void setup() {}

	/** {@inheritDoc} */
	public boolean isInterActive() {
		return false;
	}

	/** {@inheritDoc} */
	public void tearDown(Storage service) {}

	public Object getResource(String key) {
		return resource.get(key);
	}

	public void addResource(String key, Object value) {
		resource.put(key, value);
	}
	
	/**
	 * Send a info message to the log.
	 * @param message to log.
	 */
	public void log(String message) {
		logger.info(message);
	}
	
	public void log(String format, Object...args) {
		logger.info(String.format(format, args));
	}
}
