package no.skippy.agent.event;

import java.awt.geom.Point2D;
import java.util.Set;

import no.skippy.agent.Agent;
import no.skippy.sim.control.BroomControls;
import no.skippy.sim.control.ReadOnlyRockCollection;

/**
 * Event type passed to {@link Agent} after a rock has been set.
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class AfterTurnEvent extends TurnEvent {

	/** */
	private static final long serialVersionUID = 1L;
	
	/** position of the broom when rock was set.*/
	private Point2D broomPos = null;
	/** split time of the broom when rock was set.*/
	private int splitTime = 0;
	/** turn of the the broom when rock was set.*/
	private boolean isOutTurn = false;


	public AfterTurnEvent(BroomControls agentControls,
			ReadOnlyRockCollection rockCollection,
			Set<Integer> rocksInPlay, boolean isDark, int rockidx16, int round,
			int numberOfRounds, Point2D broomPos, int splitTime,
			boolean isOutTurn, boolean hasHammer, String opponent) {
		super(agentControls, rockCollection, rocksInPlay, isDark, rockidx16, round,
				numberOfRounds, hasHammer, opponent);
		
		this.broomPos = broomPos;
		this.splitTime = splitTime;
		this.isOutTurn = isOutTurn;
	}

	/**
	 * @return the broom position of the rock set.
	 */
	public Point2D previousBroom() {
		return broomPos;
	}
	
	/**
	 * @return the split time of the rock set.
	 */
	public int previousSplitTime() {
		return splitTime;
	}
	
	/**
	 * 
	 * @return true if the rock set was out turn.
	 */
	public boolean ispreviousOutTurn() {
		return isOutTurn;
	}
}
