package no.skippy.agent.event;

import java.util.Set;

import no.skippy.agent.Agent;
import no.skippy.sim.control.ReadOnlyRockCollection;

/**
 * Event type passed to {@link Agent} after a round.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class RoundEndedEvent extends Event {

	public enum Result {WIN,TIE,LOOSE}

	private ReadOnlyRockCollection rockCollection;
	private Set<Integer> rocksInPlay;
	private boolean isDark = false;
	private int round;
	private int points;
	private Result result;
	private boolean hasHammer;
	
	/**
	 * Create a RoundEnded Event.
	 * 
	 * @param rockCollection
	 *            position of all rocks when ending the round.
	 * @param rocksInPlay
	 *            list of the rock in play when ending the round.
	 * @param isDark
	 *            true if the player receiving the event is dark.
	 * @param round
	 *            what round it is.
	 * @param points
	 *            after round. >=0
	 * @param hasHammer
	 *            true if the player receiving the event had the last stone.
	 */
	public RoundEndedEvent(ReadOnlyRockCollection rockCollection,
			Set<Integer> rocksInPlay, boolean isDark, int round, int points,
			Result result, boolean hasHammer) {

		this.rockCollection = rockCollection;
		this.rocksInPlay = rocksInPlay;
		this.isDark = isDark;
		this.round = round;
		this.points = points;
		this.result = result;
		this.hasHammer = hasHammer;
	}

	/**
	 * @return collection of the position of all rocks.
	 */
	public ReadOnlyRockCollection getRockPositions() {
		return rockCollection;
	}

	/**
	 * @return list of rocks what was in play when the round ended.
	 */
	public Set<Integer> getRocksInPlay() {
		return rocksInPlay;
	}

	/**
	 * @return the round number.
	 */
	public int getRound() {
		return round;
	}

	/**
	 * @return the points. Always positive.
	 */
	public int getPoints() {
		return points;
	}
	
	/**
	 * @return clue about the result. 
	 */
	public Result getResult() {
		return result;
	}
	
	/**
	 * @return true if the player is dark (red) rock.
	 */
	public boolean isDark() {
		return isDark;
	}

	/**
	 * @return true if the player had the last stone.
	 */
	public boolean isHasHammer() {
		return hasHammer;
	}
	
}
