package no.skippy.agent.examples;

import java.awt.geom.Point2D;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import no.skippy.agent.BasicAgent;
import no.skippy.agent.event.RoundEndedEvent;
import no.skippy.agent.event.TurnEvent;
import no.skippy.agent.util.CurlingUtils;
import no.skippy.agent.util.GameLog;
import no.skippy.agent.util.Storage;

public class BullsEyeAgent extends BasicAgent {

	// features
	private static final int POINTS = 0;
	private static final int IN_HOUSE = 1;
	private static final int CORNERGUARD = 2;
	private static final int CENTERGUARD = 3;
	private static final int FEATURES_COUNT = 4;
	private static final int MIN_WEIGHT = -20;
	private static final int MAX_WEIGHT = 20;
	private static final int DARK = 0;
	private static final int LIGTH = 1;
	private static final String[] description = new String[] { "Points.",
			"Rocks in house.", "Has corderguard.", "Has centerguard." };

	String filefolder = LinearTargetAgent.class.getResource("").getFile();
	String filePath = filefolder + "BullsEye.xml";
	String logPath = filefolder + "BullsEyeLog.jskip";

	private double bestEst = 0;

	GameLog<LFunctionLog> log = new GameLog<LFunctionLog>();
	private String logEntry = "BullsEye";

	private LFunction curLFunction = new LFunction(FEATURES_COUNT, MIN_WEIGHT,
			MAX_WEIGHT);

	private List<LFunction> featureHist = new ArrayList<LFunction>();

	private double xbound = CurlingUtils.feetToMeter(5);
	private double xstep = CurlingUtils.feetToMeter(1);

	private double bestx = 0;
	private int bestSplit = 0;
	private boolean bestOutTurn = false;

	@Override
	public void onTurn(TurnEvent e) {

		int[] features = createFeatureList(e);

		bestEst = curLFunction.getEstimateV(features);

		curLFunction.setFeatures(features);

		Point2D broom = e.getBroom();
		for (int split = 3600; split > 3000; split -= (split < 3200) ? 500
				: (split < 3500 ? 50 : 200)) {

			for (double xpos = -xbound; xpos <= xbound; xpos += xstep
					+ (split < 3300 ? xstep : 0)) {
				broom.setLocation(xpos, 0);

				e.setSplitTimeMillies(split);
				e.setBroom(broom);

				e.setOutTurn(xpos < 0);

				features = createFeatureList(e);
				double est = curLFunction.getEstimateV(features);
				if (est > bestEst) {
					setBest(est, xpos, split, xpos < 0, features);
				}

				e.setOutTurn(xpos > 0);
				features = createFeatureList(e);
				est = curLFunction.getEstimateV(features);
				if (est > bestEst) {
					setBest(est, xpos, split, xpos < 0, features);
				}
			}
			// log("Split: " + split);
		}

		// if center guard => less split time || curl around guard.
		// or if corner guard => less split time || curl from opposite side.
		// if both => test all?

		// backup the feature
		curLFunction.setRoundsPlayed(curLFunction.getRoundsPlayed() + 1);
		featureHist.add(curLFunction.clone());

		setRock(e);
	}

	@Override
	public void onWin(RoundEndedEvent event) {
		trainAgent(100, curLFunction.getConstant(), event.getPoints());
	}

	@Override
	public void onTie(RoundEndedEvent event) {
		trainAgent(10, curLFunction.getConstant(), event.getPoints());
	}

	@Override
	public void onLoose(RoundEndedEvent event) {
		trainAgent(-100, curLFunction.getConstant(), event.getPoints());
	}

	private int[] createFeatureList(TurnEvent e) {
		int[] features = new int[FEATURES_COUNT];

		int[] p = CurlingUtils.getPoints(e.getAllRocks(), e.getRocksInPlay());
		features[POINTS] = Math.max(p[DARK], p[LIGTH])
				* ((p[DARK] > p[LIGTH] && e.isPlayerDark() || p[DARK] < p[LIGTH]
						&& !e.isPlayerDark()) ? 1 : -1);

		for (Integer i16 : e.getRocksInPlay()) { // InHouse
			if (CurlingUtils.isInHouse(e.getAllRocks().getCurrentPos(i16))) {
				features[IN_HOUSE] += 1 * ((e.isRockDark(i16) && e
						.isPlayerDark()) // dark&dark
						|| (!e.isRockDark(i16) && !e.isPlayerDark()) ? 1 : -1); // light&light
			}

			// guard testing
			Point2D rock = e.getCurrentPos(i16);
			// corner guard
			features[CORNERGUARD] += 1 * (CurlingUtils.isCornerGuard(rock) ? 1
					: -1);
			// center guard
			features[CENTERGUARD] += 1 * (CurlingUtils.isCenterGuard(rock) ? 1
					: -1);
		}
		return features;
	}

	private void trainAgent(int score, double constant, int rockPoints) {
		double[] weights = curLFunction.getWeights();

		log("Training using experience from " + featureHist.size() + " rounds.");
		// backtrack, and update weights.
		for (int index = featureHist.size() - 1; index >= 0; index--) {
			LFunction his = featureHist.get(index);
			his.setWeights(weights);
			weights = his.updateWeights(score, constant);
		}

		curLFunction.setWeights(weights);
		curLFunction.setTrainingCount(curLFunction.getTrainingCount() + 1);
		logLFunction();
		featureHist.clear();

		// log the progress
		log.addLog(logEntry, new LFunctionLog(weights.clone(), curLFunction
				.getConstant(), rockPoints, curLFunction.getRoundsPlayed(),
				curLFunction.getEstimateV(curLFunction.getFeatures()), score));
	}

	@Override
	public String getAgentName() {
		return "Mr. Bull - Linear Target";
	}

	@Override
	public void loadResources(Storage service) {
		if (new File(filePath).exists()) // avoid chicken and egg problem
			service.loadObject(LFunction.class, filePath, this);
	}

	@Override
	public void tearDown(Storage service) {
		service.saveResource(curLFunction, filePath);
		service.saveResource(log, logPath);
	}

	@Override
	public void setup() {
		if (getResource(filePath) != null) {
			curLFunction = (LFunction) getResource(filePath);
		}

		curLFunction.setDescription(description);
		logLFunction();
		log.addEntry(logEntry);
	}

	private void setRock(TurnEvent e) {
		e.setBroom(new Point2D.Double(bestx, 0));
		e.setOutTurn(bestOutTurn);
		e.setSplitTimeMillies(bestSplit);
	}

	private void setBest(double bestEst, double bestX, int bestSplit,
			boolean bestOutTurn, int[] features) {
		this.bestEst = bestEst;
		this.bestx = bestX;
		this.bestSplit = bestSplit;
		this.bestOutTurn = bestOutTurn;
		this.curLFunction.setFeatures(features);
	}

	private void logLFunction() {
		log("w" + 0 + ": " + curLFunction.getWeights()[0]);
		for (int i = 1; i < curLFunction.getWeights().length; i++) {
			log("w" + i + ": " + curLFunction.getWeights()[i] + "\t ("
					+ curLFunction.getDescription()[i - 1] + ")");
		}
		log("Training count: " + curLFunction.getTrainingCount());
	}
}
