package no.skippy.agent.examples;

import java.awt.geom.Point2D;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import no.skippy.agent.BasicAgent;
import no.skippy.agent.event.RoundEndedEvent;
import no.skippy.agent.event.TurnEvent;
import no.skippy.agent.util.CurlingUtils;
import no.skippy.agent.util.GameLog;
import no.skippy.agent.util.Storage;

public class LinearTargetAgent extends BasicAgent {

	private ArrayList<LFunction> lFunctions = new ArrayList<LFunction>();
	private LFunction currentLFunction = new LFunction(7, -1, 1);

	private String LOG_FILE = "agentlog/LinearTarget/LinearTarget_%s.jskip";
	private String LINEAR_DATA = "agentlog/LinearTarget/LinearTargetLearning_01.04.08_1.xml";

	private static SimpleDateFormat DATE_STAMP = new SimpleDateFormat(
			"dd.MM.yy_hhmm");

	private String logEntry = "LinearAgent";

	GameLog<LFunctionLog> log = new GameLog<LFunctionLog>();

	public void loadResources(Storage service) {
		if (new File(LINEAR_DATA).exists()) // avoid chicken and egg problem
			service.loadObject(LFunction.class, LINEAR_DATA, this);
		else {
			String[] description = { "Points", "Dark In House",
					"Light In House", "Dark Covered", "Light Covered",
					"Corner Guard", "Center Guard" };
			currentLFunction.setDescription(description);
		}
	}

	public void setup() {
		if (getResource(LINEAR_DATA) != null) {
			currentLFunction = (LFunction) getResource(LINEAR_DATA);
		}

		// for (int i = 0; i < currentLFunction.getWeights().length; i++) {
		// log("w" +i + ": " + currentLFunction.getWeights()[i]);
		// }
		logLFunction();
		log.addEntry(logEntry);
	}

	public void onLoose(RoundEndedEvent event) {
		trainAgent(-100 * event.getPoints(), currentLFunction.getConstant(), -event.getPoints());
	}

	public void onTie(RoundEndedEvent event) {
		trainAgent(10, currentLFunction.getConstant(), event.getPoints());
	}

	public void onWin(RoundEndedEvent event) {
		trainAgent(100 * event.getPoints(), currentLFunction.getConstant(), event.getPoints());
	}

	double bestEstimate = Double.NEGATIVE_INFINITY;
	double bestX = 0;
	int bestSplitTime = 1001;
	boolean bestOutTurn = false;

	public void onTurn(TurnEvent event) {

		// for some iteration over broom :
		// set broom position
		// caclulateLFunction
		// store the best result
		int[] features = calculateLFeatureList(event);
		bestEstimate = Double.NEGATIVE_INFINITY; // currentLFunction.getEstimateV(features);
		bestX = event.getBroom().getX();
		bestSplitTime = event.getSplitTimeMillies().getValue();
		bestOutTurn = event.getOutTurn();

		for (double x = -2.0; x <= 2.0; x += .5) {
			for (int splitTime = 3000; splitTime < 3700; splitTime += 100) {
				event.setBroom(new Point2D.Double(x, 0));
				event.setSplitTimeMillies(splitTime);
				event.setOutTurn(true);

				int[] currentFeatures = calculateLFeatureList(event);
				double currentEstimate = currentLFunction
						.getEstimateV(currentFeatures);

				if (currentEstimate > bestEstimate) {
					setBest(currentEstimate, x, splitTime, event.getOutTurn(),
							features);
					// bestEstimate = currentEstimate;
					// currentLFunction.setFeatures(currentFeatures);
					// bestX = x;
					// bestSplitTime = splitTime;
					// bestOutTurn = event.getOutTurn();
				}

				event.setOutTurn(false);
				currentFeatures = calculateLFeatureList(event);
				currentEstimate = currentLFunction
						.getEstimateV(currentFeatures);

				if (currentEstimate > bestEstimate) {
					setBest(currentEstimate, x, splitTime, event.getOutTurn(),
							features);
					// bestEstimate = currentEstimate;
					// currentLFunction.setFeatures(currentFeatures);
					// bestX = x;
					// bestSplitTime = splitTime;
					// bestOutTurn = event.getOutTurn();
				}
			}
		}

		currentLFunction
				.setRoundsPlayed(currentLFunction.getRoundsPlayed() + 1);
		lFunctions.add(currentLFunction.clone());

		event.setBroom(new Point2D.Double(bestX, 0));
		event.setSplitTimeMillies(bestSplitTime);
		event.setOutTurn(bestOutTurn);
	}

	public int[] calculateLFeatureList(TurnEvent event) {
		// 0 points
		// 1 darkInHouse
		// 2 lightInHouse
		// 3 darkCovered
		// 4 lightCovered
		// 5 cornerGuard
		// 6 centerGuard

		int[] features = new int[7];

		for (int rockId : event.getRocksInPlay()) {
			if (CurlingUtils.isInHouse(event.getCurrentPos(rockId))) {

				if (event.isRockDark(rockId))
					features[1]++;
				else
					features[2]++;

				for (int guardId : event.getRocksInPlay()) {
					if (guardId != rockId) {
						if ((Math
								.abs((event.getCurrentPos(rockId).getX() - event
										.getCurrentPos(guardId).getX())) < CurlingUtils
								.feetToMeter(.5))
								&& event.getCurrentPos(rockId).getY() < event
										.getCurrentPos(guardId).getY()) {
							if (event.isRockDark(rockId))
								features[3]++;
							else
								features[4]++;
						}
					}
				}

				if (CurlingUtils.isPointRock(rockId, event.getAllRocks(), event
						.getRocksInPlay())) {

					if (event.isPlayerDark() == event.isRockDark(rockId))
						features[0]++;
					else
						features[0]--;

				}
			} else if (event.getCurrentPos(rockId).getY() > 0) {
				if (Math.abs(event.getCurrentPos(rockId).getX()) < CurlingUtils
						.feetToMeter(2)) {
					features[6]++;
				} else
					features[5]++;
			}
		}

		return features;
	}

	private void trainAgent(int score, double constant, int rockPoints) {
		double[] weights = currentLFunction.getWeights();

		log("Training using experience from " + lFunctions.size() + " rounds.");
		// backtrack, and update weights.
		for (int index = lFunctions.size() - 1; index >= 0; index--) {
			LFunction his = lFunctions.get(index);
			his.setWeights(weights);

			weights = his.updateWeights(score, constant);
		}

		currentLFunction.setWeights(weights);
		currentLFunction
				.setTrainingCount(currentLFunction.getTrainingCount() + 1);
		// for (int i = 0; i < currentLFunction.getWeights().length; i++) {
		// log("w" +i + ": " + currentLFunction.getWeights()[i]);
		// }
		logLFunction();

		// currentLFunction.setConstant(1.0/(currentLFunction.getTrainingCount()*1.0));
		// currentLFunction.setConstant(currentLFunction.getConstant()-0.0005);
		lFunctions.clear();

		log.addLog(logEntry, new LFunctionLog(weights.clone(), currentLFunction
				.getConstant(), rockPoints, currentLFunction.getRoundsPlayed(),
				currentLFunction.getEstimateV(currentLFunction.getFeatures()),
				score));
	}

	private void setBest(double bestEst, double bestX, int bestSplit,
			boolean bestOutTurn, int[] features) {
		this.bestEstimate = bestEst;
		this.bestX = bestX;
		this.bestSplitTime = bestSplit;
		this.bestOutTurn = bestOutTurn;
		this.currentLFunction.setFeatures(features);
	}

	public void tearDown(Storage service) {
		String logfile = String.format(LOG_FILE, DATE_STAMP.format(new Date()));
		while (new File(logfile).exists()) {
			logfile += ".new";
		}
		service.saveResource(currentLFunction, logfile.replace("jskip", "xml"));
		service.saveResource(log, logfile);
		// log(getName() + " saved LFunction to: " + filePath);
	}
	
	public String getAgentName() {
		return "LinearTargetAgent";
	}

	private void logLFunction() {
		log("w" + 0 + ": " + currentLFunction.getWeights()[0]);
		for (int i = 1; i < currentLFunction.getWeights().length; i++) {
			log("w" + i + ": " + currentLFunction.getWeights()[i] + "\t ("
					+ currentLFunction.getDescription()[i - 1] + ")");
		}
		log("Training count: " + currentLFunction.getTrainingCount());
	}
}
