package no.skippy.agent.skippy;

import no.skippy.agent.skippy.api.Featurizer;
import no.skippy.agent.skippy.api.State;

/**
 * Implementation of the Featurizer interface.
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class AgentStateFeatures implements Featurizer {

	/** number of features.*/
	public int numFeatures = 0; 

	/** feature values.*/
	public double[] features; 

	/** feature weights.*/
	public double[] featuresWeights; 

	/** learning rate.*/
	public double learningRate; // learning rate

	/**
	 * Constructor, initiate the feature weighs with random values.
	 * @param lowerBounds for random.
	 * @param upperBounds for random.
	 * @param numFeatures number of features.
	 */
	public AgentStateFeatures(int lowerBounds, int upperBounds, int numFeatures) {
		this.numFeatures = numFeatures;
		features = new double[numFeatures];
		featuresWeights = new double[numFeatures + 1];

		for (int i = 0; i < featuresWeights.length; i++) {
			featuresWeights[i] = (Math.random() * (upperBounds - lowerBounds + 1))
					+ lowerBounds;
		}
	}

	public double[] getFeatureWeights() {
		return featuresWeights.clone();
	}

	public double[] getFeatures(State state) {
		return state.getDoubleArray();
	}

	public void setFeatureWeights(double[] newFeatureWeights) {
		if (newFeatureWeights.length != featuresWeights.length)
			throw new UnsupportedOperationException("Wrong weight array size");
		
		for (int i = 0; i < newFeatureWeights.length; i++) {
			featuresWeights[i] = newFeatureWeights[i];
		}
	}

	public double getEstimate(State state) {
		this.features = getFeatures(state);

		double estimate = featuresWeights[0];
		// features can have equal length to weights. If so, ignores features at index
		// greater than the featureWeight length.
		for (int i = 0; i < featuresWeights.length-1; i++) {
			estimate += featuresWeights[i + 1] * features[i];
		}

		return estimate;
	}

	public double getLearningRate() {
		return learningRate;
	}

	public String[] getFeatureWeightsDescription(State state) {
		return state.getDescription();
	}

	public void setLearningRate(double learningRate) {
		this.learningRate = learningRate;
	}
}
