package no.skippy.agent.skippy;

import no.skippy.agent.skippy.api.State;

public class SkippyAgentV2 extends SkippyAgent {
	
	protected void configure() {
		setAgentName("E2 SkippyAgent Normal/Hammer (V2)");
		useHammer(true);
		setNormalSuffix("26.05.09_Trained");
		setHammerSuffix("26.05.09_Trained2");
		setNumberOfActionsToCheck(DEFAULT_NUMBER_OF_ACTION);
		setWriteOut(false);
	}
	
	/**
	 * Train the weights. Using the Linear Target Function Expression (5.2, 5.3)
	 * 
	 * @param points
	 *            received.
	 * @param realValue
	 *            reward.
	 */
	protected void trainWeights(int points, double realValue) {
		log(TRAIN, stateLog.size());

		if (features.getLearningRate() == 0.0) {
			log(SKIPP_TRAINING, features.getLearningRate());

		} else {
			String message = "\n\n";
			message += format(FORMAT, "", "Estimated", "Real Value");

			State finalState = stateLog.get(stateLog
					.size() - 1);
			// update last state
			message += updateWeights(finalState, realValue);
			
			// update the previous states using their successors estimate.
			for (int index = stateLog.size()-2; index >= 0; index--) {
				State state = stateLog.get(index);
				double successorEstimate = features.getEstimate(stateLog.get(index+1));
				message += updateWeights(state, successorEstimate);
			}

			incTrainingCount(features);
			stateLog.clear();

			double trainingCountDouble = getTrainingCount(features);
			features.setLearningRate(1 / (200 + trainingCountDouble));

			log(message);
			log(AFTER_TRAINING, getTrainingCount(features), roundsPlayed);
		}

		printFeatureWeights(features, bestState, hammerRound);
		addLog(points, features.getEstimate(bestState), realValue);
	}
	
	/**
	 * @param stateEstimate estimate value of a state.
	 * @param successorEstimate estimate value of the successor state.
	 * @param message log.
	 * @return updated message.
	 */
	private String updateWeights(State state, double successorEstimate) {
		String message = "";
		double stateEstimate = features.getEstimate(state);
		double[] weights = features.getFeatureWeights();
			for (int i = 1; i < weights.length; i++) {
				weights[i] = features.getFeatureWeights()[i]
						+ (features.getLearningRate() * (successorEstimate - stateEstimate))
						* features.getFeatures(state)[i - 1];
			}
			
			features.setFeatureWeights(weights);
			message += format(FORMAT, "", DIGIT.format(stateEstimate), DIGIT
					.format(successorEstimate));
		return message;
	}

}
