package no.skippy.agent.skippy.rlearning;

import java.util.Arrays;

import no.skippy.agent.event.TurnEvent;
import no.skippy.agent.skippy.AgentState;

/**
 * State definition used by {@link RLSkippyAgent}. Uses discrete only discrete
 * values, and includes the additional state feature rockNumber.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class RLAgentState extends AgentState {

	/** The rock in a round. (0-7). */
	public int rockNumber;

	/**
	 * Constructor for {@link RLAgentState}.
	 * @param points positive or negative.
	 * @param myRocksInHouse number of rocks.
	 * @param notmyRocksInHouse number of rocks.
	 * @param myRocksCovered (0+)
	 * @param notmyRocksCovered (0+)
	 * @param cornerGuard (0+)
	 * @param centerGuard (0+)
	 * @param rockNumber (0-7)
	 */
	public RLAgentState(int points, int myRocksInHouse, int notmyRocksInHouse,
			int myRocksCovered, int notmyRocksCovered, int cornerGuard,
			int centerGuard, int rockNumber) {
		super(points, myRocksInHouse, notmyRocksInHouse, myRocksCovered,
				notmyRocksCovered, cornerGuard, centerGuard);
		
		this.myRocksInHouse = myRocksInHouse;
		this.notmyRocksInHouse = notmyRocksInHouse;
		this.rockNumber = rockNumber;
	}
	
	/**
	 * Constructor.
	 * @param state to copy, uses only the discrete feature.
	 * @param rockNumber the rock number.
	 */
	public RLAgentState(AgentState state, int rockNumber) {
		super();
		double[] features = state.getDoubleArray();
		this.points = (int) features[0];
		this.myRocksInHouse = (int) features[1];
		this.notmyRocksInHouse = (int) features[2];
		this.myRocksCovered = (int) features[3];
		this.notmyRocksCovered = (int) features[4];
		this.cornerGuard = (int) features[5];
		this.centerGuard = (int) features[6];
		
		this.myRocksInHouse = discreteMyRocksInHouse;
		this.notmyRocksInHouse = discreteNotMyRocksInHouse;
		this.rockNumber = rockNumber;
	}
	
	/**
	 * Construct a {@link RLAgentState} using a turn event.
	 * 
	 * @param event
	 *            to convert to state.
	 * @param rockNumber
	 *            the rock number.
	 */
	public RLAgentState(TurnEvent event, int rockNumber) {
		super(event);
		this.myRocksInHouse = discreteMyRocksInHouse;
		this.notmyRocksInHouse = discreteNotMyRocksInHouse;
		this.rockNumber = rockNumber;
	}

	public double[] getDoubleArray() {
		double[] stateArray = { points, myRocksInHouse, notmyRocksInHouse,
				myRocksCovered, notmyRocksCovered, cornerGuard, centerGuard,
				rockNumber };
		return stateArray;
	}

	public String[] getDescription() {
		String[] descriptionArray = { "Constant.", "Match Points.",
				"Num. of Rocks that is mine and in house.",
				"Num. of Rocks that is NOT mine and in house.",
				"Num. of Rocks that is mine and covered.",
				"Num. of Rocks that is NOT mine and covered.",
				"Num. of Cornerguards.", "Num. of Centerguards.",
				"In rock number in round. (of 8)" };
		return descriptionArray;
	}

	public String toString() {
		return super.toString() + "\nRocks played. " + rockNumber;
	}

	public boolean equals(Object that) {
		if (this == that) return true;
		if (!(that instanceof RLAgentState)) {
			return false;
		}

		RLAgentState thats = ((RLAgentState) that);
		return Arrays.equals(getDoubleArray(), thats.getDoubleArray());
	}
	
	public int hashCode() {
		int hash = 7;
		double[] ll = getDoubleArray();
		for (double d : ll)
			hash = (int) (31 * hash + d);
		return hash;
	}
}
