package no.skippy.agent.skippy.rlearning.db;

public class RowEntry {

	/** SQL Table key. */
	protected int actionID = -1;

	/** SQL Table key. */
	protected String stateID = "";

	/** its Q-value Q(stateID, actionID). */
	protected double qvalue = 0.0;

	/** Q-value reward. */
	protected double reward = 0.0;

	/** Number of times the Q(s,a) has been used. */
	protected int countUsed = 0;

	/**
	 * Empty Constructor.
	 */
	public RowEntry() {}

	/**
	 * Create a entry for Q(state,action).
	 * 
	 * @param stateID
	 *            to use.
	 * @param actionID
	 *            to use.
	 */
	public RowEntry(String stateID, int actionID) {
		this.stateID = stateID;
		this.actionID = actionID;
	}

	/**
	 * Get the values of a Q table.
	 * 
	 * @return the values of the row.
	 */
	public Object[] values() {
		return new Object[] { stateID, actionID, qvalue, reward, countUsed };
	}

	/**
	 * Get the SET SQL command.
	 * 
	 * @return SQL <code>SET .... </code>
	 */
	public String getSet() {
		return String.format("qvalue='%s', reward='%s', countUsed='%s'",
				qvalue, reward, countUsed);
	}

	/**
	 * @return the Q(s,a).
	 */
	public double getQvalue() {
		return qvalue;
	}

	/**
	 * @param qvalue
	 *            to set.
	 */
	public void setQvalue(double qvalue) {
		this.qvalue = qvalue;
	}

	/**
	 * @return the reward assigned to Q(s,a).
	 */
	public double getReward() {
		return reward;
	}

	/**
	 * @param reward
	 *            to set.
	 */
	public void setReward(double reward) {
		this.reward = reward;
	}

	/**
	 * @return times Q(s,a) has been used.
	 */
	public int getCountUsed() {
		return countUsed;
	}

	/**
	 * @param countUsed
	 *            to set.
	 */
	public void setCountUsed(int countUsed) {
		this.countUsed = countUsed;
	}

	/**
	 * @return actionID of Q(s,a).
	 */
	public int getActionID() {
		return actionID;
	}

	/**
	 * @return stateID of Q(s,a).
	 */
	public String getStateID() {
		return stateID;
	}

	/**
	 * Determine if this is the Q-table row.
	 * 
	 * @param stateID
	 *            to use.
	 * @param actionID
	 *            to use.
	 * @return true if stateID and actionID are the same.
	 */
	public boolean belongsTo(String stateID, int actionID) {
		return this.stateID.equals(stateID) && this.actionID == actionID;
	}
}
