package no.skippy.agent.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Class for logging a collection of objects.
 *<p>
 * Example:<br>
 * GameLog<AClass> log = new GameLog<AClass>();<br>
 * log.addEntry("a");<br>
 * log.addLog("a", aclassobject);
 * </p>
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class GameLog<E> {

	private Map<String, List<E>> entries = new HashMap<String, List<E>>();

	/**
	 * Create a new log entry.
	 * 
	 * @param key
	 *            log name.
	 */
	public void addEntry(String key) {
		entries.put(key, new ArrayList<E>());
	}

	/**
	 * Add a log item to a log entry.
	 * 
	 * @param key
	 *            of the log entry.
	 * @param item
	 *            to add.
	 */
	public void addLog(String key, E item) {
		entries.get(key).add(item);
	}

	/**
	 * @return whole log.
	 */
	public Map<String, List<E>> getEntries() {
		return entries;
	}

	/**
	 * @param entries
	 *            to set.
	 */
	public void setEntries(Map<String, List<E>> entries) {
		this.entries = entries;
	}

}
