package no.skippy.sim.control;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import no.skippy.sim.model.RockCollection;

import org.apache.commons.logging.Log;
import org.jcurl.core.api.RockSet;
import org.jcurl.core.api.RockType.Pos;
import org.jcurl.core.log.JCLoggerFactory;
import org.jcurl.core.ui.ChangeManager;
import org.jcurl.core.ui.PosMemento;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;

public class MouseRockHandler extends PBasicInputEventHandler {

	private static Log log = JCLoggerFactory.getLogger(MouseRockHandler.class);

	private final RockCollection model;

	private ChangeManager changeManager = null;

	public MouseRockHandler(RockCollection model) {
		this.model = model;
		addMouseHandlers();
	}

	private void addMouseHandlers() {
		for (PNode node : model.getInitialRocks()) {
			node.addInputEventListener(this);
		}
	}

	/**
	 * V -&gt; M Controller. Push rock movements via {@link ChangeManager} to
	 * the model.
	 */
	private PosMemento first = null;

	private PNode climb(PNode node) {
		for (;;) {
			if (node.getPickable()
					&& node.getAttribute(RockCollection.ATTR_IDX16) != null)
				return node;
			node = node.getParent();
			if (node == null)
				return null;
		}
	}

	@SuppressWarnings("unchecked")
	private PosMemento create(final PInputEvent e, final PNode node) {
		final RockSet<Pos> rs = (RockSet<Pos>) node
				.getAttribute(RockCollection.ATTR_ROCKSET);
		final Point2D p = model.getWorld().globalToLocal(e.getPosition());
		if (log.isDebugEnabled())
			log.debug(p);
		return new PosMemento(rs, (Integer) node
				.getAttribute(RockCollection.ATTR_IDX16), p);
	}

	@Override
	public void mouseDragged(final PInputEvent e) {
		final PNode node = climb(e.getPickedNode());
		if (log.isDebugEnabled())
			log.debug(node.getAttribute(RockCollection.ATTR_IDX16));
		getChanger().temporary(create(e, node));
		e.setHandled(true);
	}

	@Override
	public void mousePressed(final PInputEvent e) {
		final PNode node = climb(e.getPickedNode());
		if (log.isDebugEnabled())
			log.debug(node.getAttribute(RockCollection.ATTR_IDX16));
		final int i16 = (Integer) node.getAttribute(RockCollection.ATTR_IDX16);
		final RockSet<Pos> ipos = model.getCurves().getInitialPos();
		first = new PosMemento(ipos, i16, ipos.getRock(i16).p());
		e.setHandled(true);
		log.info("Pressed " + i16 + ": "
				+ model.getCurves().getCurrentPos().getRock(i16).p().getX()
				+ "\t"
				+ model.getCurves().getCurrentPos().getRock(i16).p().getY());
		// test remove
		List<Integer> ll = new ArrayList<Integer>();
		for (int i = 0; i < 16; i++)
			ll.add((i));
	}

	@Override
	public void mouseReleased(final PInputEvent e) {
		final PNode node = climb(e.getPickedNode());
		if (log.isDebugEnabled())
			log.debug(node.getAttribute(RockCollection.ATTR_IDX16));
		getChanger().undoable(first, create(e, node));
		e.setHandled(true);
	}

	private ChangeManager getChanger() {
		return ChangeManager.getTrivial(changeManager);
	}
}
