package no.skippy.sim.control;

import java.awt.geom.Point2D;

import org.jcurl.core.api.Rock;
import org.jcurl.core.api.RockType.Pos;

import no.skippy.agent.BasicAgent;
import no.skippy.sim.model.RockCollection;

/**
 * Wrapper class for the {@link RockCollection}, used to provide 
 * read only access to the RockSet for {@link BasicAgent}.
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class ReadOnlyRockCollection {
	
	private RockCollection collection;
	public ReadOnlyRockCollection(RockCollection collection) {
		this.collection = collection;
	}

	/**
	 * @param i16 rock to use.
	 * @return the current position of the rock.
	 */
	public Point2D getCurrentPos(int i16) {
		Rock<Pos> pos = collection.getCurves().getCurrentPos().getRock(i16);
		return new Point2D.Double(pos.getX(), pos.getY()) {
			/** */
			private static final long serialVersionUID = 1L;
			@Override
			public void setLocation(double x, double y) {
				throw new UnsupportedOperationException("read-only");
			}
			@Override
			public void setLocation(Point2D p) {
				throw new UnsupportedOperationException("read-only");
			}
		};
	}
	
	/**
	 * 
	 * @param i16 rock to use.
	 * @return the position of the rock before (changed by broom).
	 */
	public Point2D getInitialPos(int i16) {
		Rock<Pos> pos = collection.getCurves().getInitialPos().getRock(i16);
		
		return new Point2D.Double(pos.getX(), pos.getY()) {
			/** */
			private static final long serialVersionUID = 1L;

			@Override
			public void setLocation(double x, double y) {
				throw new UnsupportedOperationException("read-only");
			}
			
			@Override
			public void setLocation(Point2D p) {
				throw new UnsupportedOperationException("read-only");
			}
		};
	}
}
