package no.skippy.sim.control;

import org.jcurl.core.api.Rock;
import org.jcurl.core.ui.BroomPromptModel;

/**
 * Interface used by the {@link RockBroomControl} to update the {@link Rock}
 * 's after an agent has set the broom.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */

public interface RockSender {

	/**
	 * Send the rock. And update all involved rocks.
	 * 
	 * @param current
	 *            is the current ix16, the brooms
	 *            {@link BroomPromptModel#getIdx16()}
	 * @param next
	 *            is the ix16 that the broom should use next. </P> Equal to
	 *            calling {@link RockSender#setRock(int, int, int, boolean)} with
	 *            interactive equal to false.
	 * @param rockCount
	 *            number of rock set in the round.
	 */
	void setRock(int current, int next, int rockCount);

	/**
	 * Send rock and move to the next.
	 * 
	 * @param current
	 *            ix16.
	 * @param next
	 *            1x16.
	 * @param rockCount
	 *            number of rock set in the round.
	 * @param interactive
	 *            if true, the call will wait for mouse input by the agent.
	 */
	void setRock(int current, int next, int rockCount, boolean interactive);

	/**
	 * 
	 * @return true if the {@link RockSender#setRock(int, int, int, boolean)} with
	 *         true, has been called and no response from the agent has been
	 *         received. i.e. the rock has not been set.
	 */
	boolean isWaiting();

	/**
	 * Reset all the rock's position to their respective Home's.
	 * 
	 * @param idx16
	 *            rock {@link BroomPromptModel#setIdx16(int)}.
	 */
	void reset(int idx16);
}
