package no.skippy.sim.control;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import no.skippy.agent.BasicAgent;
import no.skippy.agent.skippy.rlearning.QLearning;
import no.skippy.agent.util.Storage;
import no.skippy.sim.util.LoadFromXMLTask;
import no.skippy.sim.util.SaveToXMLTask;

import org.apache.commons.logging.Log;
import org.jcurl.core.log.JCLoggerFactory;
import org.jdesktop.application.Application;
import org.jdesktop.application.Task;

/**
 * Implementation of the {@link Storage} interface. Class supports saving and
 * loading classes using XML as representation.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class StorageService implements Storage {

	private Application application;

	private List<Task<Object, Void>> loadtaskQueue = new ArrayList<Task<Object, Void>>();
	private List<Task<Void, Void>> savetaskQueue = new ArrayList<Task<Void, Void>>();

	private static final Log log = JCLoggerFactory.getLogger(QLearning.class);
	
	public StorageService(Application app) {
		if (app == null) {
			throw new UnsupportedOperationException("Application can't be null");
		}
		this.application = app;
	}

	public void saveResource(Object obj, String filePath) {
		saveResource(obj, filePath, null);
	}

	public void saveResource(Object obj, String filePath, String id) {
		SaveToXMLTask saveTask = new SaveToXMLTask(application, new File(
				filePath), obj, id);
		savetaskQueue.add(saveTask);
	}

	public void loadObject(Class<?> cl, String filePath, BasicAgent owner) {
		loadObject(cl, filePath, null, owner);
	}

	public void loadObject(Class<?> cl, String filePath, String nameId,
			BasicAgent owner) {
		
		if (!new File(filePath).exists()) {
			log.error("Could not load file:" + filePath);
			return;
		}
		
		LoadFromXMLTask loadTask = new LoadFromXMLTask(application, filePath,
				cl, nameId, owner);
		loadtaskQueue.add(loadTask);
	}

	/**
	 * Executes all queued save task.
	 * 
	 * @see SaveToXMLTask
	 */
	protected void saveResources() {
		for (Task<Void, Void> task : savetaskQueue) {
			application.getContext().getTaskService().execute(task);
			while (!task.isCancelled() && !task.isDone())
				try {
					Thread.sleep(20);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
		}
		savetaskQueue.clear();
	}

	/**
	 * Execute all queued load task.
	 * 
	 * @see LoadFromXMLTask
	 */
	protected void loadResource() {
		for (Task<Object, Void> task : loadtaskQueue) {
			application.getContext().getTaskService().execute(task);
			while (!task.isCancelled() && !task.isDone())
				try {
					Thread.sleep(20);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
		}
		loadtaskQueue.clear();
	}
}
