package no.skippy.sim.control;

import java.awt.geom.Rectangle2D;

import javax.swing.ActionMap;
import javax.swing.JMenu;

import no.skippy.sim.util.GuiCreator;
import no.skippy.sim.util.Zoom;
import no.skippy.sim.view.SimulatorView;

import org.jcurl.core.api.IceSize;
import org.jcurl.core.api.RockProps;
import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ApplicationContext;
import org.jdesktop.application.Resource;
import org.jdesktop.application.ResourceMap;

/**
 * Collection of zoom actions.
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class ZoomActions {
	
	@Resource
	private int ZOOM_SLOW; // milli's, see properties 
	@Resource
	private int ZOOM_FAST; // milli's, see properties

	/** all actions in this object. */
	private ActionMap actionMap;
	/** view controller ref. for {@link SimView}.*/
	private SimulatorView view;
	
	private ResourceMap resourceMap;
	
	public static final Rectangle2D HousePlus;
	
	static {
		final double r2 = 2 * RockProps.DEFAULT.getRadius();
		final double x = IceSize.SIDE_2_CENTER + r2;
		HousePlus = new Rectangle2D.Double(-x, -(IceSize.HOG_2_TEE + r2),
				2 * x, IceSize.HOG_2_TEE + IceSize.BACK_2_TEE + 3 * r2 + 2 * r2);
	}
	
	/**
	 * Default Constructor.
	 * @param simView for the SimView.
	 */
	public ZoomActions(SimulatorView simView) {
		this.view = simView;
		ApplicationContext appContext = Application.getInstance().getContext();
		actionMap = appContext.getActionMap(this);
		resourceMap = appContext.getResourceMap(ZoomActions.class);
		resourceMap.injectFields(this);
	}
	
	/**
	 * Create a menu containing all zoom actions.
	 * @return a window menu. {@link JMenu}.
	 */
	public JMenu getMenu() {
		final String[] viewMenuActionNames = { 
				"viewHouse", "view12Foot",
				"viewComplete", "viewActive", "---", "viewZoomIn",
				"viewZoomOut", "---", "viewPanNorth", "viewPanSouth",
				"viewPanEast", "viewPanWest" };

		JMenu menu = GuiCreator.createMenu("viewMenu", viewMenuActionNames,
				this.actionMap);
		
		resourceMap.injectComponents(menu);
		return menu;
	}
	
//	rest is Actions...
	
	@Action
	public void viewMenu() {
//		empty!! used to retrieve menu name.
	}

	/** View Menu Action */
	@Action
	public void view12Foot() {
		Zoom.zoom(view, Zoom.TwelvePlus, ZOOM_SLOW);
	}

	/** View Menu Action */
	@Action
	public void viewActive() {
		Zoom.zoom(view, Zoom.ActivePlus, ZOOM_SLOW);
	}

	/** View Menu Action */
	@Action
	public void viewComplete() {
		Zoom.zoom(view, Zoom.CompletePlus, ZOOM_SLOW);
	}

	/** View Menu Action */
	@Action
	public void viewHouse() {
		Zoom.zoom(view, Zoom.HousePlus, ZOOM_SLOW);
	}

	/** View Menu Action */
	@Action
	public void viewPanEast() {
		Zoom.pan(view, 0.2, 0, ZOOM_FAST);
	}

	/** View Menu Action */
	@Action
	public void viewPanNorth() {
		Zoom.pan(view, 0, -0.2, ZOOM_FAST);
	}

	/** View Menu Action */
	@Action
	public void viewPanSouth() {
		Zoom.pan(view, 0, 0.2, ZOOM_FAST);
	}

	/** View Menu Action */
	@Action
	public void viewPanWest() {
		Zoom.pan(view, -0.2, 0, ZOOM_FAST);
	}

	/** View Menu Action */
	@Action
	public void viewZoomIn() {
		Zoom.zoom(view, null, 0.75, ZOOM_FAST);
	}

	/** View Menu Action */
	@Action
	public void viewZoomOut() {
		Zoom.zoom(view, null, 1.25, ZOOM_FAST);
	}

	public int getZOOM_SLOW() {
		return ZOOM_SLOW;
	}

	public int getZOOM_FAST() {
		return ZOOM_FAST;
	}
}
