package no.skippy.sim.model;

import java.awt.geom.AffineTransform;
import java.util.IdentityHashMap;
import java.util.Map;

import no.skippy.sim.control.SimulatorViewController;
import no.skippy.sim.view.SimulatorView;

import org.jcurl.core.api.ComputedTrajectorySet;
import org.jcurl.core.api.Rock;
import org.jcurl.core.api.RockSet;
import org.jcurl.core.api.RockType.Pos;
import org.jcurl.zui.piccolo.PIceFactory;
import org.jcurl.zui.piccolo.PRockFactory;

import edu.umd.cs.piccolo.PNode;

/**
 * RocksModel, a model for the {@link SimulatorView} and {@link SimulatorViewController}.
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class RockCollection {
//  TODO: move to resource.
	private static final int major = 255;
	private static final int minor = 64;
//	TODO: move to resource.
	public static final String ATTR_IDX16 = "idx16";
	public static final String ATTR_ROCK = "rock";
	public static final String ATTR_ROCKSET = "rockset";
	public static final String ATTR_TRIGGER_CURVE_UPDATE = "trigger_curve_update";

	private final Map<Rock<Pos>, PNode> r2n = new IdentityHashMap<Rock<Pos>, PNode>();
	private ComputedTrajectorySet curves = null;

	private final PNode[] path         = new PNode[RockSet.ROCKS_PER_SET];
	private final PNode[] currentRocks = new PNode[RockSet.ROCKS_PER_SET];
	private final PNode[] initialRocks = new PNode[RockSet.ROCKS_PER_SET];;

	private final PNode world = new PNode();
	private final PNode rocks = new PNode();
	
	public RockCollection() {
		initialize();
	}
	
	private void initialize() {
		world.addChild(new PIceFactory.Fancy().newInstance());

		final PNode initRock = new PNode();
		final PNode currRock = new PNode();
		final PNode paths = new PNode();

		final PRockFactory initRockFactory = new PRockFactory.Fancy(minor);
		final PRockFactory currRockFactory = new PRockFactory.Fancy(major);

		for (int index = RockSet.ROCKS_PER_SET - 1; index >= 0; index--) {
			PNode node;
			initRock.addChild(node = initialRocks[index] = initRockFactory
					.newInstance(index));
			node.setPickable(true);
			
			currRock.addChild(node = currentRocks[index] = currRockFactory
					.newInstance(index));
			node.addAttribute(ATTR_TRIGGER_CURVE_UPDATE, true);
			node.setChildrenPickable(false);
			
//			node.addPropertyChangeListener( new RockInPlayListener(this));
			paths.addChild(path[index] = new PNode());
		}
		
		rocks.addChild(paths);
		rocks.addChild(initRock);
		rocks.addChild(currRock);
		rocks.setVisible(true);
		
		world.addChild(rocks);
		world.setTransform(AffineTransform.getScaleInstance(1, -1));
	}

	// getters / setters?
	public Map<Rock<Pos>, PNode> getRocNodeMap() {
		return r2n;
	}

	public ComputedTrajectorySet getCurves() {
		return curves;
	}
	
	public void setCurves(ComputedTrajectorySet curves) {
		this.curves = curves;
	}

	public PNode[] getPath() {
		return path;
	}
	
	public PNode getWorld() {
		return world;
	}
	
	public PNode getRocks() {
		return rocks;
	}

	public PNode[] getInitialRocks() {
		return initialRocks;
	}

	public PNode[] getCurrentRocks() {
		return currentRocks;
	}
}
