package no.skippy.sim.model;

import java.awt.geom.Point2D;

import org.jcurl.core.api.Rock;
import org.jcurl.core.api.RockSet;
import org.jcurl.core.api.Unit;
import org.jcurl.core.api.RockType.Pos;

public class RockComparable implements Comparable<RockComparable> {
	
	private double distance;
	private int rockId;


	public RockComparable(int rockId, Rock<Pos> rockPos) {
		this.rockId = rockId;
		distance = computeDistance(rockPos.getX(), rockPos.getY());
	}
	
	public RockComparable(int rockId, Point2D rockPos) {
		this.rockId = rockId;
		distance = computeDistance(rockPos.getX(), rockPos.getY());
	}

	private double computeDistance(double x, double y) {
		return Math.sqrt(x*x + y*y);
	}

	public int compareTo(RockComparable o) {
		double diff = this.distance - o.getDistance();
		if (diff < 0) { return -1; }
		else if (diff > 0) { return 1; }
		else return 0;
	}
	
	public double getDistance() {
		return distance;
	}
	
	public boolean inHouse() {
		if (distance <= Unit.f2m(6.5)) { return true; }
		else { return false; }
	}
	
	public String toString() {
		return rockId + " Distance: " + distance + "Dark?: " + this.isDark();
	}
	public boolean isDark() {
		return RockSet.isDark(rockId);
	}
}
