package no.skippy.sim.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import no.skippy.agent.Agent;
import no.skippy.agent.BasicAgent;

import org.apache.commons.logging.Log;
import org.jcurl.core.log.JCLoggerFactory;
import org.jdesktop.application.Application;
import org.jdesktop.application.Task;

import com.thoughtworks.xstream.XStream;

/**
 * Load a class from XML.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class LoadFromXMLTask extends Task<Object, Void> {

	private static Log logger = JCLoggerFactory
			.getLogger(LoadFromXMLTask.class);

	/** location of the XML resource file. */
	private String filePath;

	/** Class type of the resource. */
	private Class<?> cl;

	/** Resource name. */
	private String name;

	/** Resource owner. */
	private Agent owner;

	/**
	 * 
	 * @param application
	 *            ref.
	 * @param file
	 *            to use.
	 * @param cl
	 *            type to load.
	 * @param name
	 *            XML alias used.
	 * @param owner
	 *            to add the object loaded.
	 */
	public LoadFromXMLTask(Application application, String file, Class<?> cl,
			String name, BasicAgent owner) {
		super(application);
		this.filePath = file;
		this.cl = cl;
		this.name = name;
		this.owner = owner;
	}

	@Override
	protected Object doInBackground() throws Exception {
		logger.info("read object from: " + filePath);

		File file = new File(filePath);

		int fileLength = (int) file.length();
		int nChars = -1;
		// progress updates after every blockSize chars read
		int blockSize = Math.max(1024, fileLength / 100);

		int p = blockSize;
		char[] buffer = new char[32];
		StringBuilder contents = new StringBuilder();
		BufferedReader rdr = new BufferedReader(new FileReader(file));
		while (!isCancelled() && (nChars = rdr.read(buffer)) != -1) {
			contents.append(buffer, 0, nChars);
			if (contents.length() > p) {
				p += blockSize;
				setProgress(contents.length(), 0, fileLength);
			}
		}

		XStream stream = new XStream();
		stream.alias(name, cl);
		Object obj = stream.fromXML(contents.toString());

		if (!isCancelled()) {
			if (owner != null) {
				owner.addResource(filePath, obj);
			}
			return obj;
		} else {
			return null;
		}
	}
}
