package no.skippy.sim.util;

import java.net.URL;

import no.skippy.sim.control.SimulatorApp;

import org.apache.commons.logging.Log;
import org.jcurl.core.api.ComputedTrajectorySet;
import org.jcurl.core.api.TrajectorySet;
import org.jcurl.core.io.IONode;
import org.jcurl.core.io.IOTrajectories;
import org.jcurl.core.io.JCurlSerializer;
import org.jcurl.core.log.JCLoggerFactory;
import org.jdesktop.application.Task;

/**
 * Load a curling {@link JCurlSerializer} {@link TrajectorySet} into the
 * {@link SimulatorApp}
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class LoadJCurlResource extends Task<Void, Void> {

	private static final Log log = JCLoggerFactory
			.getLogger(LoadJCurlResource.class);
	private final URL url;
	private SimulatorApp app;
	private double currentTime;

	public LoadJCurlResource(SimulatorApp application, URL url,
			double currentTime) {
		super(application);
		this.url = url;
		this.currentTime = currentTime;
		app = application;
	}

	protected final URL getDocumentURL() {
		return url;
	}

	protected final double getCurrentTime() {
		return currentTime;
	}

	@Override
	protected Void doInBackground() throws Exception {
		log.debug(getDocumentURL());

		app.getChangeListenerManager().remove(
				app.getSimView().getRockCollection().getCurves());

		final ComputedTrajectorySet cts;
		if (getDocumentURL() == null)
			cts = null;
		else {
			final IONode node = new JCurlSerializer().read(getDocumentURL());
			final IOTrajectories it = (IOTrajectories) node;
			final TrajectorySet ts = it.trajectories().get(0);
			cts = (ComputedTrajectorySet) ts;
		}

		if (cts != null) {
			cts.setCurrentTime(getCurrentTime());
		}

		app.getViewController().setCurves(cts);
		app.getChangeListenerManager().add(cts);
		return null;
	}
}
