package no.skippy.sim.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import org.apache.commons.logging.Log;
import org.jcurl.core.log.JCLoggerFactory;
import org.jdesktop.application.Application;
import org.jdesktop.application.Task;

import com.thoughtworks.xstream.XStream;

/**
 * Save a class object to XML.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class SaveToXMLTask extends Task<Void, Void> {

	private static Log logger = JCLoggerFactory.getLogger(SaveToXMLTask.class);

	/** output file. */
	private File file;

	/** instance to save to XML file. */
	private Object obj;

	/** Alias a Class to a shorter name to be used in XML elements. */
	private String name;

	/**
	 * 
	 * @param application
	 *            ref.
	 * @param file
	 *            path to use.
	 * @param obj
	 *            to save.
	 * @param name
	 *            XML object alias.
	 */
	public SaveToXMLTask(Application application, File file, Object obj,
			String name) {
		super(application);
		this.file = file;
		this.obj = obj;
		this.name = name;
	}

	@Override
	protected Void doInBackground() throws Exception {
		XStream stream = new XStream();
		if (name != null) {
			stream.alias(name, obj.getClass());
		}

		String xmlOfObj = stream.toXML(obj);

		File tmpFile = file;
		tmpFile.createNewFile();

		BufferedWriter out = null;
		int fileLength = xmlOfObj.length();
		int blockSize = Math.max(1024, 1 + ((fileLength - 1) / 100));

		try {
			out = new BufferedWriter(new FileWriter(tmpFile));
			int offset = 0;
			while (!isCancelled() && (offset < fileLength)) {
				int length = Math.min(blockSize, fileLength - offset);
				out.write(xmlOfObj, offset, length);
				offset += blockSize;
				setProgress(Math.min(offset, fileLength), 0, fileLength);
			}
		} finally {
			if (out != null) {
				out.close();
			}
		}
		return null;
	}

	protected void finished() {
		logger.debug("wrote object to: " + file.getAbsolutePath());
	}
}
