package no.skippy.sim.util;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;


import no.skippy.sim.control.Zoomable;

import org.jcurl.core.api.IceSize;
import org.jcurl.core.api.RockProps;
import org.jcurl.core.api.Unit;

/**
 * Utility class. Handling zooming of a {@link Zoomable} object.
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class Zoom {

	public static final Rectangle2D ActivePlus;
	/** All from back to back */
	public static final Rectangle2D CompletePlus;
	/** House area plus 1 rock margin plus "out" rock space. */
	public static final Rectangle2D HousePlus;
	/** 12-foot circle plus 1 rock */
	public static final Rectangle2D TwelvePlus;

	static {
		final double r2 = 2 * RockProps.DEFAULT.getRadius();
		final double x = IceSize.SIDE_2_CENTER + r2;
		HousePlus = new Rectangle2D.Double(-x, -(IceSize.HOG_2_TEE + r2),
				2 * x, IceSize.HOG_2_TEE + IceSize.BACK_2_TEE + 3 * r2 + 2 * r2);
		final double c12 = r2 + Unit.f2m(6.0);
		TwelvePlus = new Rectangle2D.Double(-c12, -c12, 2 * c12, 2 * c12);
		ActivePlus = new Rectangle2D.Double(-x, -(IceSize.HOG_2_HOG
				+ IceSize.HOG_2_TEE + r2), 2 * x, IceSize.HOG_2_HOG
				+ IceSize.HOG_2_TEE + IceSize.BACK_2_TEE + 3 * r2 + 2 * r2);
		CompletePlus = new Rectangle2D.Double(-x, -(IceSize.HOG_2_TEE
				+ IceSize.HOG_2_HOG + IceSize.HACK_2_HOG + r2), 2 * x,
				IceSize.HOG_2_HOG + 2 * IceSize.HACK_2_HOG);
	}

	public static void pan(final Zoomable dst, final double rx, final double ry,
			final int dt) {
		if (dst == null)
			return;
		final RectangularShape src = dst.getZoom();
		zoom(dst, new Rectangle2D.Double(src.getX() + src.getWidth() * rx, src
				.getY()
				+ src.getHeight() * ry, src.getWidth(), src.getHeight()), dt);
	}

	public static void zoom(final Zoomable dst, final Point2D center,
			final double ratio, final int dt) {
		if (dst == null)
			return;
		final RectangularShape src = dst.getZoom();
		final double w = src.getWidth() * ratio;
		final double h = src.getHeight() * ratio;
		final double cx, cy;
		if (center == null) {
			cx = src.getCenterX();
			cy = src.getCenterY();
		} else {
			cx = center.getX();
			cy = center.getY();
		}
		zoom(dst, new Rectangle2D.Double(cx - w / 2, cy - h / 2, Math.abs(w),
				Math.abs(h)), dt);
	}

	public static void zoom(final Zoomable dst, final RectangularShape viewport,
			final int dt) {
		if (dst == null)
			return;
		dst.setZoom(viewport, dt);
	}

}
