package no.skippy.sim.view;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import org.jdesktop.application.Application;

import no.skippy.sim.model.SimulatorScore;

/**
 * Display round score in a {@link JList}.
 */
public class ScorePanel extends JPanel {

	/** */
	private static final long serialVersionUID = 1L;

	private JList scoreList;
	private ListModel model;

	private JLabel scoreTotalLabel;

	private static String format = "Totals: (red): %s   (yellow): %s.";

	/** dark player total score. */
	int darkScore = 0;

	/** light player total score. */
	int lightScore = 0;

	/**
	 * Constructor.
	 * @param model to use in {@link JList}.
	 */
	public ScorePanel(ListModel model) {
		this.model = model;
		initComponents();
		model.addListDataListener(new ScoreListener());
	}

	public void initComponents() {

		if (Application.getInstance() != null) {
			format = Application.getInstance().getContext().getResourceMap()
					.getString("ScorePanel.scoreTotalLabel.text");
		}

		scoreTotalLabel = new JLabel(String.format(format, 0, 0));

		setBackground(Color.white);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		scoreList = new JList(model);
		scoreList.setEnabled(false);

		JScrollPane scrollPane = new JScrollPane() {
			/** */
			private static final long serialVersionUID = 1L;

			public Dimension getMaximumSize() {
				return getPreferredSize();
			}

			public Dimension getMinimumSize() {
				return getPreferredSize();
			}
		};
		scrollPane.setViewportView(scoreList);
		scrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
		scrollPane.setPreferredSize(new Dimension(200, 190));

		add(scrollPane);
		JPanel status = new JPanel();
		status.setLayout(new BoxLayout(status, BoxLayout.X_AXIS));
		status.setBackground(getBackground());
		status.add(scoreTotalLabel);
		add(status);
	}

	class ScoreListener implements ListDataListener {

		public void contentsChanged(ListDataEvent e) {}

		public void intervalAdded(ListDataEvent e) {
			final int index = model.getSize() - 1;

			darkScore += ((SimulatorScore) model.getElementAt(index))
					.getDarkScore();
			lightScore += ((SimulatorScore) model.getElementAt(index))
					.getLightScore();
			scoreTotalLabel.setText(String
					.format(format, darkScore, lightScore));
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					scoreList.setSelectedIndex(index); // select new item added.
					scoreList.ensureIndexIsVisible(index);
				}
			});
		}

		public void intervalRemoved(ListDataEvent e) {
			scoreTotalLabel.setText(String.format(format, 0, 0));
			darkScore = 0;
			lightScore = 0;
		}
	}
}
