package no.skippy.sim.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;

import javax.swing.JComponent;

import no.skippy.sim.control.Zoomable;
import no.skippy.sim.model.RockCollection;

import org.jcurl.core.ui.TrajectoryBroomPromptWrapper;
import org.jcurl.zui.piccolo.BroomPromptSimple;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.util.PPaintContext;

/**
 * Curling Ice view. Display the rocks + broom.
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class SimulatorView extends JComponent implements Zoomable {

	private static final int DEFAULT_TRANSITION_MILLIS = 333;
	/** */
	private static final long serialVersionUID = 1L;

	private RockCollection rocksModel;

	private PCanvas canvas;

	protected transient RectangularShape tmpViewPort = null;

	private final BroomPromptSimple simpleBroom = new BroomPromptSimple();

	public SimulatorView(RockCollection rocks,
			TrajectoryBroomPromptWrapper broomModel) {
		this.rocksModel = rocks;
		simpleBroom.setModel(broomModel);
		initComponents();
	}

	private void initComponents() {

		rocksModel.getRocks().addChild(simpleBroom); // add the broom to the ice
		canvas = new PCanvas();
		canvas.setAnimatingRenderQuality(PPaintContext.HIGH_QUALITY_RENDERING);
		canvas
				.setInteractingRenderQuality(PPaintContext.HIGH_QUALITY_RENDERING);

		setLayout(new BorderLayout());
		add(canvas, BorderLayout.CENTER);

		canvas.getLayer().addChild(getRockCollection().getWorld());
		canvas.setBackground(super.getBackground());
		setVisible(true);
	}

	@Override
	public void doLayout() {
		super.doLayout();
		if (tmpViewPort != null)
			setZoom(tmpViewPort, 0);
	}

	// getters setters
	public RectangularShape getViewPort() {
		return tmpViewPort;
	}

	public PLayer getIceLayer() {
		return canvas.getLayer();
	}

	public void setViewPort(RectangularShape viewPort) {
		this.tmpViewPort = viewPort;
	}

	public PCanvas getCanvas() {
		return canvas;
	}

	public RockCollection getRockCollection() {
		return rocksModel;
	}

	@Override
	public void setBackground(final Color backgrondColor) {
		canvas.setBackground(backgrondColor);
		super.setBackground(backgrondColor);
	}

	public TrajectoryBroomPromptWrapper getBroomModel() {
		return (TrajectoryBroomPromptWrapper) simpleBroom.getModel();
	}

	// Zoomable interface implementation.

	/**
	 * change view port zoom. shortcut for calling
	 * {@link SimulatorView#setZoom(RectangularShape, int)} with -1.
	 */
	public void setZoom(RectangularShape viewport) {
		setZoom(getViewPort(), -1);
	}

	public void setZoom(RectangularShape viewport, int transitionMillis) {
		if (transitionMillis < 0) {
			transitionMillis = DEFAULT_TRANSITION_MILLIS;
		}
		final Rectangle2D r;
		if (viewport instanceof Rectangle2D)
			r = (Rectangle2D) viewport;
		else
			r = new Rectangle2D.Double(viewport.getX(), viewport.getY(),
					viewport.getWidth(), viewport.getHeight());
		setViewPort((Rectangle2D) r.clone());
		getCanvas().getCamera().animateViewToCenterBounds(r, true,
				transitionMillis);
	}

	public RectangularShape getZoom() {
		return getViewPort();
	}

}
