package no.skippy.sim.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSlider;

import org.jdesktop.application.TaskMonitor;

/**
 * A StatusBar panel that tracks a TaskMonitor. </p> This class is based on
 * example code in AppFramework.
 * 
 * @see <a href="https://appframework.dev.java.net/intro/index.html">Swing
 *      Application Framework</a>.
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class StatusBar extends JPanel implements PropertyChangeListener {

	/** */
	private static final long serialVersionUID = -1942095421852625601L;

	private static final int MIN_PROGRESS = 5;
	private static final int MAX_PROGRESS = 100;

	private final JLabel messageLabel;
	private final JProgressBar progressBar;

	private final int rspace = 10;
	private final Color topBorder = Color.gray.brighter();

	public StatusBar(TaskMonitor taskMonitor, Action action, JSlider speed) {
		super(new GridBagLayout());
		setBackground(Color.white);
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		setBackground(Color.white);

		messageLabel = new JLabel();
		progressBar = new JProgressBar(MIN_PROGRESS, MAX_PROGRESS);

//		progressBar.setEnabled(false);

		messageLabel.setMaximumSize(new Dimension(3400, 16));
		messageLabel.setMinimumSize(new Dimension(10, 16));
		messageLabel.setBorder(BorderFactory.createEmptyBorder(0, rspace, 0, 0));
		add(messageLabel);

		add(new JButton(action));
		speed.setMaximumSize(new Dimension(190, 29));
		speed.setMinimumSize(new Dimension(190, 29));
		add(speed);

		progressBar.setMinimum(100);
		progressBar.setMaximumSize(new Dimension(146, 20));
		progressBar.setMinimumSize(new Dimension(146, 20));
		progressBar.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, rspace));
		add(progressBar);
		taskMonitor.addPropertyChangeListener(this);
	}

	public void propertyChange(PropertyChangeEvent evt) {
		String propertyName = evt.getPropertyName();
		if ("started".equals(propertyName)) {
			progressBar.setEnabled(true);
			progressBar.setIndeterminate(true);
		} else if ("done".equals(propertyName)) {
			progressBar.setIndeterminate(false);
			progressBar.setEnabled(false);
			progressBar.setValue(0);
		} else if ("message".equals(propertyName)) {
			String text = (String) (evt.getNewValue());
			setMessage(text);
		} else if ("progress".equals(propertyName)) {
			int value = (Integer) (evt.getNewValue());
			progressBar.setEnabled(true);
			progressBar.setIndeterminate(false);
			progressBar.setValue(value);
		}
	}

	public void setMessage(String message) {
		messageLabel.setText((message == null) ? "" : message);
	}

	// paint top border
	public void paintChildren(Graphics g) {
		super.paintChildren(g);

		final Graphics2D g2 = (Graphics2D) g;
		final int w = getWidth();
		g2.setPaint(topBorder);
		g2.fillRect(0, 0, w, 1);
	}

}
