package no.skippy.sim.control;

import static org.junit.Assert.*;

import java.awt.geom.Point2D;
import java.net.URL;

import no.skippy.sim.model.RockCollection;

import org.jcurl.core.api.ComputedTrajectorySet;
import org.jcurl.core.api.RockSet;
import org.jcurl.core.api.RockType.Pos;
import org.junit.Before;
import org.junit.Test;

/**
 * Test the Read-only {@link RockCollection}. </p>
 * <ul>
 * <li>Test for getting the right positions.</li>
 * <li>Test that the positions are read-only.</li>
 * </ul>
 * 
 * @author <a href="mailto:frode.aa@gmail.com">Frode Aannevik </a>
 */
public class ReadOnlyRockCollectionTest {

	private RockCollection rockCollection;
	private ReadOnlyRockCollection readOnlyRocks;
	private static String red_out = "red_out.jcz";

	@Before
	public void setUp() throws Exception {
		this.rockCollection = new RockCollection();
		this.readOnlyRocks = new ReadOnlyRockCollection(rockCollection);
		ComputedTrajectorySet redOut = LoadJCurl.load(getResource(red_out));
		new SimulatorViewController(null, rockCollection).setCurves(redOut);
	}

	@Test
	public void testGetCurrentPos() {
		RockSet<Pos> expected = rockCollection.getCurves().getCurrentPos();
		for (int i = 0; i < RockSet.ROCKS_PER_SET; i++) {
			assertEquals(expected.getRock(i).p(), readOnlyRocks
					.getCurrentPos(i));
		}
	}

	@Test
	public void testGetInitialPos() {
		RockSet<Pos> expected = rockCollection.getCurves().getInitialPos();
		for (int i = 0; i < RockSet.ROCKS_PER_SET; i++) {
			assertEquals(expected.getRock(i).p(), readOnlyRocks
					.getCurrentPos(i));
		}
	}

	@Test(expected = UnsupportedOperationException.class)
	public void testCurrentRights() {
		for (int i = 0; i < RockSet.ROCKS_PER_SET; i++) {
			readOnlyRocks.getCurrentPos(i)
					.setLocation(new Point2D.Double(2, 3));
		}
	}

	@Test(expected = UnsupportedOperationException.class)
	public void testCurrentRightsVariation() {
		for (int i = 0; i < RockSet.ROCKS_PER_SET; i++) {
			readOnlyRocks.getCurrentPos(i).setLocation(3, 2);
		}
	}

	@Test(expected = UnsupportedOperationException.class)
	public void testInitialRights() {
		for (int i = 0; i < RockSet.ROCKS_PER_SET; i++) {
			readOnlyRocks.getInitialPos(i)
					.setLocation(new Point2D.Double(2, 3));
		}
	}

	@Test(expected = UnsupportedOperationException.class)
	public void testInitialRightsVariation() {
		for (int i = 0; i < RockSet.ROCKS_PER_SET; i++) {
			readOnlyRocks.getInitialPos(i).setLocation(2, 3);
		}
	}

	private URL getResource(String filename) {
		return SimulatorApp.class.getResource("resources/" + filename);
	}

}
